@@ -33,7 +33,7 @@ from openpower.consts import SVP64CROffs
 from openpower.decoder.power_svp64 import SVP64RM, decode_extra
 
 from openpower.decoder.isa.radixmmu import RADIX
-from openpower.decoder.isa.mem import Mem, swap_order
+from openpower.decoder.isa.mem import Mem, swap_order, MemException
 
 from collections import namedtuple
 import math
@@ -54,6 +54,11 @@ special_sprs = {
 REG_SORT_ORDER = {
     # TODO (lkcl): adjust other registers that should be in a particular order
     # probably CA, CA32, and CR
+    "FRT": 0,
+    "FRA": 0,
+    "FRB": 0,
+    "FRC": 0,
+    "FRS": 0,
     "RT": 0,
     "RA": 0,
     "RB": 0,
@@ -70,6 +75,8 @@ REG_SORT_ORDER = {
     "overflow": 1,
 }
 
+fregs = ['FRA', 'FRB', 'FRC', 'FRS', 'FRT']
+
 
 def create_args(reglist, extra=None):
     retval = list(OrderedSet(reglist))
@@ -358,6 +365,10 @@ def get_pdecode_idx_in(dec2, name):
                                      in2, in2_isvec)
     print ("get_pdecode_idx_in in3", name, in3_sel, In3Sel.RS.value,
                                      in3, in3_isvec)
+    print ("get_pdecode_idx_in FRS in3", name, in3_sel, In3Sel.FRS.value,
+                                     in3, in3_isvec)
+    print ("get_pdecode_idx_in FRC in3", name, in3_sel, In3Sel.FRC.value,
+                                     in3, in3_isvec)
     # identify which regnames map to in1/2/3
     if name == 'RA':
         if (in1_sel == In1Sel.RA.value or
@@ -380,6 +391,20 @@ def get_pdecode_idx_in(dec2, name):
             return in2, in2_isvec
         if in3_sel == In3Sel.RS.value:
             return in3, in3_isvec
+    elif name == 'FRA':
+        if in1_sel == In1Sel.FRA.value:
+            return in1, in1_isvec
+    elif name == 'FRB':
+        if in2_sel == In2Sel.FRB.value:
+            return in2, in2_isvec
+    elif name == 'FRC':
+        if in3_sel == In3Sel.FRC.value:
+            return in3, in3_isvec
+    elif name == 'FRS':
+        if in1_sel == In1Sel.FRS.value:
+            return in1, in1_isvec
+        if in3_sel == In3Sel.FRS.value:
+            return in3, in3_isvec
     return None, False
 
 
@@ -402,7 +427,7 @@ def get_pdecode_cr_out(dec2, name):
     if name == 'CR0':
         if out_sel == CROutSel.CR0.value:
             return out, o_isvec
-    print ("get_pdecode_idx_out not found", name)
+    print ("get_pdecode_cr_out not found", name)
     return None, False
 
 
@@ -422,6 +447,15 @@ def get_pdecode_idx_out(dec2, name):
                                       OutSel.RT_OR_ZERO.value, out, o_isvec)
         if out_sel == OutSel.RT.value:
             return out, o_isvec
+    elif name == 'FRA':
+        print ("get_pdecode_idx_out", out_sel, OutSel.FRA.value, out, o_isvec)
+        if out_sel == OutSel.FRA.value:
+            return out, o_isvec
+    elif name == 'FRT':
+        print ("get_pdecode_idx_out", out_sel, OutSel.FRT.value,
+                                      OutSel.FRT.value, out, o_isvec)
+        if out_sel == OutSel.FRT.value:
+            return out, o_isvec
     print ("get_pdecode_idx_out not found", name)
     return None, False
 
@@ -441,7 +475,9 @@ class ISACaller:
     def __init__(self, decoder2, regfile, initial_sprs=None, initial_cr=0,
                  initial_mem=None, initial_msr=0,
                  initial_svstate=0,
-                 initial_insns=None, respect_pc=False,
+                 initial_insns=None,
+                 fpregfile=None,
+                 respect_pc=False,
                  disassembly=None,
                  initial_pc=0,
                  bigendian=False,
@@ -456,6 +492,8 @@ class ISACaller:
             initial_sprs = {}
         if initial_mem is None:
             initial_mem = {}
+        if fpregfile is None:
+            fpregfile = [0] * 32
         if initial_insns is None:
             initial_insns = {}
             assert self.respect_pc == False, "instructions required to honor pc"
@@ -487,6 +525,7 @@ class ISACaller:
             initial_svstate = SVP64State(initial_svstate)
         self.svstate = initial_svstate
         self.gpr = GPR(decoder2, self, self.svstate, regfile)
+        self.fpr = GPR(decoder2, self, self.svstate, fpregfile)
         self.spr = SPR(decoder2, initial_sprs) # initialise SPRs before MMU
         self.mem = Mem(row_bytes=8, initial_mem=initial_mem)
         self.imem = Mem(row_bytes=4, initial_mem=initial_insns)
@@ -521,6 +560,7 @@ class ISACaller:
         self.namespace = {}
         self.namespace.update(self.spr)
         self.namespace.update({'GPR': self.gpr,
+                               'FPR': self.fpr,
                                'MEM': self.mem,
                                'SPR': self.spr,
                                'memassign': self.memassign,
@@ -545,12 +585,29 @@ class ISACaller:
         self.decoder = decoder2.dec
         self.dec2 = decoder2
 
+    def call_trap(self, trap_addr, trap_bit):
+        """calls TRAP and sets up NIA to the new execution location.
+        next instruction will begin at trap_addr.
+        """
+        self.TRAP(trap_addr, trap_bit)
+        self.namespace['NIA'] = self.trap_nia
+        self.pc.update(self.namespace, self.is_svp64_mode)
+
     def TRAP(self, trap_addr=0x700, trap_bit=PIb.TRAP):
+        """TRAP> saves PC, MSR (and TODO SVSTATE), and updates MSR
+
+        TRAP function is callable from inside the pseudocode itself,
+        hence the default arguments.  when calling from inside ISACaller
+        it is best to use call_trap()
+        """
         print("TRAP:", hex(trap_addr), hex(self.namespace['MSR'].value))
         # store CIA(+4?) in SRR0, set NIA to 0x700
         # store MSR in SRR1, set MSR to um errr something, have to check spec
+        # store SVSTATE (if enabled) in SVSRR0
         self.spr['SRR0'].value = self.pc.CIA.value
         self.spr['SRR1'].value = self.namespace['MSR'].value
+        if self.is_svp64_mode:
+            self.spr['SVSRR0'] = self.namespace['SVSTATE'].value
         self.trap_nia = SelectableInt(trap_addr, 64)
         self.spr['SRR1'][trap_bit] = 1  # change *copy* of MSR in SRR1
 
@@ -782,7 +839,16 @@ class ISACaller:
             code = self.disassembly[self._pc]
             print("sim-execute", hex(self._pc), code)
         opname = code.split(' ')[0]
-        yield from self.call(opname)
+        try:
+            yield from self.call(opname)         # execute the instruction
+        except MemException as e:                # check for memory errors
+            if e.args[0] != 'unaligned':         # only doing aligned at the mo
+                raise e                          # ... re-raise
+            # run a Trap but set DAR first
+            print ("memory unaligned exception, DAR", e.dar)
+            self.spr['DAR'] = SelectableInt(e.dar, 64)
+            self.call_trap(0x600, PIb.PRIV)                # 0x600, privileged
+            return
 
         # don't use this except in special circumstances
         if not self.respect_pc:
@@ -794,10 +860,12 @@ class ISACaller:
         # TODO, asmregs is from the spec, e.g. add RT,RA,RB
         # see http://bugs.libre-riscv.org/show_bug.cgi?id=282
         dec_insn = yield self.dec2.e.do.insn
+        insn_1_11 = yield self.dec2.e.do.insn[1:11]
         asmcode = yield self.dec2.dec.op.asmcode
-        print("get assembly name asmcode", asmcode, hex(dec_insn))
-        asmop = insns.get(asmcode, None)
         int_op = yield self.dec2.dec.op.internal_op
+        print("get assembly name asmcode", asmcode, int_op,
+                            hex(dec_insn), bin(insn_1_11))
+        asmop = insns.get(asmcode, None)
 
         # sigh reconstruct the assembly instruction name
         if hasattr(self.dec2.e.do, "oe"):
@@ -885,9 +953,7 @@ class ISACaller:
               self.msr[MSRb.PR])
         # check MSR priv bit and whether op is privileged: if so, throw trap
         if instr_is_privileged and self.msr[MSRb.PR] == 1:
-            self.TRAP(0x700, PIb.PRIV)
-            self.namespace['NIA'] = self.trap_nia
-            self.pc.update(self.namespace, self.is_svp64_mode)
+            self.call_trap(0x700, PIb.PRIV)
             return
 
         # check halted condition
@@ -907,9 +973,7 @@ class ISACaller:
 
         if illegal:
             print("illegal", name, asmop)
-            self.TRAP(0x700, PIb.ILLEG)
-            self.namespace['NIA'] = self.trap_nia
-            self.pc.update(self.namespace, self.is_svp64_mode)
+            self.call_trap(0x700, PIb.ILLEG)
             print("name %s != %s - calling ILLEGAL trap, PC: %x" %
                   (name, asmop, self.pc.CIA.value))
             return
@@ -1026,7 +1090,10 @@ class ISACaller:
             self.namespace[regname] = regnum
             if not self.is_svp64_mode or not pred_src_zero:
                 print('reading reg %s %s' % (name, str(regnum)), is_vec)
-                reg_val = self.gpr(regnum)
+                if name in fregs:
+                    reg_val = self.fpr(regnum)
+                else:
+                    reg_val = self.gpr(regnum)
             else:
                 print('zero input reg %s %s' % (name, str(regnum)), is_vec)
                 reg_val = 0
@@ -1132,11 +1199,18 @@ class ISACaller:
                                                      is_vec)
                         output = SelectableInt(0, 256)
                     else:
-                        print('writing reg %d %s' % (regnum, str(output)),
+                        if name in fregs:
+                            ftype = 'fpr'
+                        else:
+                            ftype = 'gpr'
+                        print('writing %s %s %s' % (regnum, ftype, str(output)),
                                                      is_vec)
                     if output.bits > 64:
                         output = SelectableInt(output.value, 64)
-                    self.gpr[regnum] = output
+                    if name in fregs:
+                        self.fpr[regnum] = output
+                    else:
+                        self.gpr[regnum] = output
 
         # check if it is the SVSTATE.src/dest step that needs incrementing
         # this is our Sub-Program-Counter loop from 0 to VL-1
