@@ -236,6 +236,11 @@ class PowerOp:
         if False:
             print(row.keys())
         asmcode = row['comment']
+        # process the comment field, strip out "equals" for FP
+        if "=" in asmcode:
+            asmcode = asmcode.split("=")[-1]
+            print ("asmcode stripping =", asmcode,
+                    asmcode in asmidx, hasattr(self, "asmcode"))
         if hasattr(self, "asmcode") and asmcode in asmidx:
             res.append(self.asmcode.eq(asmidx[asmcode]))
         for bit in single_bit_flags:
@@ -410,22 +415,29 @@ class PowerDecoder(Elaboratable):
 
     def handle_subdecoders(self, switch_case, submodules, d):
         eqs = []
-        for dec in d.subdecoders:
-            if isinstance(dec, list):  # XXX HACK: take first pattern
-                dec = dec[0]
-            #print("subdec", dec.pattern, self.pname)
-            mname = get_pname("dec%d" % dec.pattern, self.pname)
-            subdecoder = PowerDecoder(self.width, dec,
-                                      name=mname,
-                                      col_subset=self.col_subset,
-                                      row_subset=self.row_subsetfn)
-            if not subdecoder.tree_analyse():  # doesn't do anything
-                del subdecoder
-                continue                      # skip
-            submodules[mname] = subdecoder
-            eqs.append(subdecoder.opcode_in.eq(self.opcode_in))
-            switch_case[dec.pattern] = self.op.eq(subdecoder.op)
-            self.actually_does_something = True
+        for dlist in d.subdecoders:
+            if not isinstance(dlist, list):  # XXX HACK: take first pattern
+                dlist = [dlist]
+            for dec in dlist:
+                #print("subdec", dec.pattern, self.pname)
+                mname = get_pname("dec%d" % dec.pattern, self.pname)
+                if mname in submodules:
+                    # sigh, HACK...
+                    mname += "_1"
+                    assert mname not in submodules
+                subdecoder = PowerDecoder(self.width, dec,
+                                          name=mname,
+                                          col_subset=self.col_subset,
+                                          row_subset=self.row_subsetfn)
+                print ("subdecoder", mname, subdecoder)
+                if not subdecoder.tree_analyse():  # doesn't do anything
+                    print ("analysed, DELETING", mname)
+                    del subdecoder
+                    continue                      # skip
+                submodules[mname] = subdecoder
+                eqs.append(subdecoder.opcode_in.eq(self.opcode_in))
+                switch_case[dec.pattern] = self.op.eq(subdecoder.op)
+                self.actually_does_something = True
 
         return eqs
 
@@ -529,11 +541,13 @@ class TopPowerDecoder(PowerDecoder):
 ####################################################
 # PRIMARY FUNCTION SPECIFYING THE FULL POWER DECODER
 
-def create_pdecode(name=None, col_subset=None, row_subset=None):
+def create_pdecode(name=None, col_subset=None, row_subset=None,
+                   include_fp=False):
     """create_pdecode - creates a cascading hierarchical POWER ISA decoder
 
     subsetting of the PowerOp decoding is possible by setting col_subset
     """
+    print ("create_pdecode", name, col_subset, row_subset, include_fp)
 
     # some alteration to the CSV files is required for SV so we use
     # a class to do it
@@ -545,9 +559,11 @@ def create_pdecode(name=None, col_subset=None, row_subset=None):
     m19.append(Subdecoder(pattern=19, opcodes=get_csv("minor_19.csv"),
                           opint=True, bitsel=(1, 11), suffix=None,
                           subdecoders=[]))
-    m19.append(Subdecoder(pattern=19, opcodes=get_csv("minor_19_00000.csv"),
-                          opint=True, bitsel=(1, 6), suffix=None,
-                          subdecoders=[]))
+    # XXX problem with sub-decoders (can only handle one),
+    # sort this another time
+    #m19.append(Subdecoder(pattern=19, opcodes=get_csv("minor_19_00000.csv"),
+    #                      opint=True, bitsel=(1, 6), suffix=None,
+    #                      subdecoders=[]))
 
     # minor opcodes.
     pminor = [
@@ -564,6 +580,19 @@ def create_pdecode(name=None, col_subset=None, row_subset=None):
                    opint=True, bitsel=(1, 5), suffix=None, subdecoders=[]),
     ]
 
+    # FP 63L/H decoders. TODO: move mffsfamily to separate subdecoder
+    if include_fp:
+        pminor.append(
+            Subdecoder(pattern=63, opcodes=get_csv("minor_63.csv"),
+                                 opint=False, bitsel=(1, 11), suffix=None,
+                                 subdecoders=[]),
+            )
+        pminor.append(
+            Subdecoder(pattern=59, opcodes=get_csv("minor_59.csv"),
+                                 opint=False, bitsel=(1, 11), suffix=None,
+                                 subdecoders=[]),
+            )
+
     # top level: extra merged with major
     dec = []
     opcodes = get_csv("major.csv")
@@ -584,11 +613,13 @@ if __name__ == '__main__':
 
         def rowsubsetfn(opcode, row):
             print("row_subset", opcode, row)
-            return row['unit'] == 'ALU'
+            return row['unit'] == 'FPU'
 
         pdecode = create_pdecode(name="rowsub",
-                                 col_subset={'function_unit', 'in1_sel'},
-                                 row_subset=rowsubsetfn)
+                                 col_subset={'opcode', 'function_unit',
+                                             'form'},
+                                 row_subset=rowsubsetfn,
+                                 include_fp=True)
         vl = rtlil.convert(pdecode, ports=pdecode.ports())
         with open("row_subset_decoder.il", "w") as f:
             f.write(vl)
@@ -602,7 +633,7 @@ if __name__ == '__main__':
 
     # full decoder
 
-    pdecode = create_pdecode()
+    pdecode = create_pdecode(include_fp=True)
     vl = rtlil.convert(pdecode, ports=pdecode.ports())
     with open("decoder.il", "w") as f:
         f.write(vl)
