@@ -81,6 +81,7 @@ class Function(Enum):
     MMU = 1 << 11
     SV = 1 << 12  # Simple-V https://libre-soc.org/openpower/sv
     VL = 1 << 13  # setvl
+    FPU = 1 << 14  # FPU
 
 
 @unique
@@ -209,35 +210,70 @@ class SVP64sat(Enum):
 # supported instructions: make sure to keep up-to-date with CSV files
 # just like everything else
 _insns = [
-    "NONE", "add", "addc", "addco", "adde", "addeo", "addi", "addic", "addic.",
-    "addis", "addme", "addmeo", "addo", "addze", "addzeo", "and", "andc",
-    "andi.", "andis.", "attn", "b", "bc", "bcctr", "bclr", "bctar",
-    "bpermd", "cmp", "cmpb", "cmpeqb", "cmpi", "cmpl", "cmpli", "cmprb",
-    "cntlzd", "cntlzw", "cnttzd", "cnttzw", "crand", "crandc", "creqv",
-    "crnand", "crnor", "cror", "crorc", "crxor", "darn", "dcbf", "dcbst",
-    "dcbt", "dcbtst", "dcbz", "divd", "divde", "divdeo", "divdeu",
+    "NONE", "add", "addc", "addco", "adde", "addeo",
+    "addi", "addic", "addic.", "addis",
+    "addme", "addmeo", "addo", "addze", "addzeo",
+    "and", "andc", "andi.", "andis.",
+    "attn",
+    "b", "bc", "bcctr", "bclr", "bctar",
+    "bpermd",
+    "cmp", "cmpb", "cmpeqb", "cmpi", "cmpl", "cmpli", "cmprb",
+    "cntlzd", "cntlzw", "cnttzd", "cnttzw",
+    "crand", "crandc", "creqv",
+    "crnand", "crnor", "cror", "crorc", "crxor",
+    "darn",
+    "dcbf", "dcbst", "dcbt", "dcbtst", "dcbz",
+    "divd", "divde", "divdeo", "divdeu",
     "divdeuo", "divdo", "divdu", "divduo", "divw", "divwe", "divweo",
-    "divweu", "divweuo", "divwo", "divwu", "divwuo", "eqv", "extsb",
-    "extsh", "extsw", "extswsli", "hrfid", "icbi", "icbt", "isel", "isync",
-    "lbarx", "lbz", "lbzu", "lbzux", "lbzx", "ld", "ldarx", "ldbrx",
-    "ldu", "ldux", "ldx", "lha", "lharx", "lhau", "lhaux", "lhax",
-    "lhbrx", "lhz", "lhzu", "lhzux", "lhzx", "lwa", "lwarx", "lwaux",
-    "lwax", "lwbrx", "lwz", "lwzcix", "lwzu", "lwzux", "lwzx", "mcrf", "mcrxr",
-    "mcrxrx", "mfcr/mfocrf", "mfmsr", "mfspr", "modsd", "modsw", "modud",
-    "moduw", "mtcrf/mtocrf", "mtmsr", "mtmsrd", "mtspr", "mulhd", "mulhdu",
-    "mulhw", "mulhwu", "mulld", "mulldo", "mulli", "mullw", "mullwo",
-    "nand", "neg", "nego", "nop", "nor", "or", "orc", "ori", "oris",
-    "popcntb", "popcntd", "popcntw", "prtyd", "prtyw", "rfid", "rldcl",
-    "rldcr", "rldic", "rldicl", "rldicr", "rldimi", "rlwimi", "rlwinm",
-    "rlwnm", "setb",
+    "divweu", "divweuo", "divwo", "divwu", "divwuo",
+    "eqv",
+    "extsb", "extsh", "extsw", "extswsli",
+    "fadd", "fadds", "fsub", "fsubs",                   # FP add / sub
+    "fcfids", "fcfidus", "fsqrts", "fres", "frsqrtes",  # FP stuff
+    "fmsubs", "fmadds", "fnmsubs", "fnmadds",           # FP 3-arg
+    "fmul", "fmuls", "fdiv", "fdivs",                   # FP mul / div
+    "fmr", "fabs", "fnabs", "fneg", "fcpsgn",           # FP move/abs/neg
+    "hrfid", "icbi", "icbt", "isel", "isync",
+    "lbarx", "lbz", "lbzu", "lbzux", "lbzx",            # load byte
+    "ld", "ldarx", "ldbrx", "ldu", "ldux", "ldx",       # load double
+    "lfs", "lfsx", "lfsu", "lfsux",                     # FP load single
+    "lfd", "lfdx", "lfdu", "lfdux", "lfiwzx", "lfiwax", # FP load double
+    "lha", "lharx", "lhau", "lhaux", "lhax",            # load half
+    "lhbrx", "lhz", "lhzu", "lhzux", "lhzx",            # more load half
+    "lwa", "lwarx", "lwaux", "lwax", "lwbrx",           # load word
+    "lwz", "lwzcix", "lwzu", "lwzux", "lwzx",           # more load word
+    "mcrf", "mcrxr", "mcrxrx", "mfcr/mfocrf",           # CR mvs
+    "mfmsr", "mfspr",
+    "modsd", "modsw", "modud", "moduw",
+    "mtcrf/mtocrf", "mtmsr", "mtmsrd", "mtspr",
+    "mulhd", "mulhdu", "mulhw", "mulhwu", "mulld", "mulldo",
+    "mulli", "mullw", "mullwo",
+    "nand", "neg", "nego",
+    "nop",
+    "nor", "or", "orc", "ori", "oris",
+    "popcntb", "popcntd", "popcntw",
+    "prtyd", "prtyw",
+    "rfid",
+    "rldcl", "rldcr", "rldic", "rldicl", "rldicr", "rldimi",
+    "rlwimi", "rlwinm",    "rlwnm",
+    "setb",
     "setvl",  # https://libre-soc.org/openpower/sv/setvl
-    "sim_cfg", "slbia", "sld", "slw", "srad", "sradi", "sraw",
-    "srawi", "srd", "srw", "stb", "stbcix", "stbcx", "stbu", "stbux", "stbx",
-    "std", "stdbrx", "stdcx", "stdu", "stdux", "stdx", "sth", "sthbrx", "sthcx",
-    "sthu", "sthux", "sthx", "stw", "stwbrx", "stwcx", "stwu", "stwux",
-    "stwx", "subf", "subfc", "subfco", "subfe", "subfeo", "subfic",
-    "subfme", "subfmeo", "subfo", "subfze", "subfzeo", "sync", "td",
-    "tdi", "tlbie", "tlbiel", "tw", "twi", "xor", "xori", "xoris",
+    "sim_cfg",
+    "slbia", "sld", "slw", "srad", "sradi",
+    "sraw", "srawi", "srd", "srw",
+    "stb", "stbcix", "stbcx", "stbu", "stbux", "stbx",
+    "std", "stdbrx", "stdcx", "stdu", "stdux", "stdx",
+    "stfs", "stfsx", "stfsu", "stfux",                  # FP store single
+    "stfd", "stfdx", "stfdu", "stfdux", "stfiwx",       # FP store double
+    "sth", "sthbrx", "sthcx", "sthu", "sthux", "sthx",
+    "stw", "stwbrx", "stwcx", "stwu", "stwux", "stwx",
+    "subf", "subfc", "subfco", "subfe", "subfeo", "subfic",
+    "subfme", "subfmeo", "subfo", "subfze", "subfzeo",
+    "sync",
+    "td", "tdi",
+    "tlbie", "tlbiel",
+    "tw", "twi",
+    "xor", "xori", "xoris",
 ]
 
 # two-way lookup of instruction-to-index and vice-versa
@@ -326,6 +362,8 @@ class MicrOp(Enum):
     OP_MTMSR = 74
     OP_TLBIE = 75
     OP_SETVL = 76
+    OP_FPOP = 77  # temporary: replace with actual ops
+    OP_FPOP_I = 78 # temporary: replace with actual ops
 
 
 @unique
@@ -335,6 +373,8 @@ class In1Sel(Enum):
     RA_OR_ZERO = 2
     SPR = 3
     RS = 4  # for some ALU/Logical operations
+    FRA = 5
+    FRS = 6
 
 
 @unique
@@ -353,6 +393,7 @@ class In2Sel(Enum):
     CONST_SH32 = 11
     SPR = 12
     RS = 13  # for shiftrot (M-Form)
+    FRB = 14
 
 
 @unique
@@ -360,6 +401,8 @@ class In3Sel(Enum):
     NONE = 0
     RS = 1
     RB = 2  # for shiftrot (M-Form)
+    FRS = 3
+    FRC = 4
 
 
 @unique
@@ -369,6 +412,7 @@ class OutSel(Enum):
     RA = 2
     SPR = 3
     RT_OR_ZERO = 4
+    FRT = 5
 
 
 @unique
