@@ -23,6 +23,8 @@ import ast
 
 # Helper function
 
+regs = ['RA', 'RS', 'RB', 'RC', 'RT']
+fregs = ['FRA', 'FRS', 'FRB', 'FRC', 'FRT']
 
 def Assign(autoassign, assignname, left, right, iea_mode):
     names = []
@@ -187,13 +189,18 @@ def identify_sint_mul_pattern(p):
     return True  # yippee!
 
 
-def apply_trailer(atom, trailer):
+def apply_trailer(atom, trailer, read_regs):
     if trailer[0] == "TLIST":
         # assume depth of one
-        atom = apply_trailer(atom, trailer[1])
+        atom = apply_trailer(atom, trailer[1], read_regs)
         trailer = trailer[2]
     if trailer[0] == "CALL":
         #p[0] = ast.Expr(ast.Call(p[1], p[2][1], []))
+        for arg in trailer[1]:
+            if isinstance(arg, ast.Name):
+                name = arg.id
+                if name in regs + fregs:
+                    read_regs.add(name)
         return ast.Call(atom, trailer[1], [])
         # if p[1].id == 'print':
         #    p[0] = ast.Printnl(ast.Tuple(p[2][1]), None, None)
@@ -205,6 +212,11 @@ def apply_trailer(atom, trailer):
         subs = trailer[1]
         if len(subs) == 1:
             idx = subs[0]
+            if isinstance(idx, ast.Name) and idx.id in regs + fregs:
+                read_regs.add(idx.id)
+            if isinstance(idx, ast.Name) and idx.id in regs:
+                print ("single atom subscript, underscored", idx.id)
+                idx = ast.Name("_%s" % idx.id, ast.Load())
         else:
             idx = ast.Slice(subs[0], subs[1], None)
         # if isinstance(atom, ast.Name) and atom.id == 'CR':
@@ -259,11 +271,14 @@ class PowerParser:
     def __init__(self, form, include_carry_in_write=False):
         self.include_ca_in_write = include_carry_in_write
         self.gprs = {}
-        form = self.sd.sigforms[form]
-        print(form)
-        formkeys = form._asdict().keys()
+        if form is not None:
+            form = self.sd.sigforms[form]
+            print(form)
+            formkeys = form._asdict().keys()
+        else:
+            formkeys = []
         self.declared_vars = set()
-        for rname in ['RA', 'RB', 'RC', 'RT', 'RS']:
+        for rname in regs + fregs:
             self.gprs[rname] = None
             self.declared_vars.add(rname)
         self.available_op_fields = set()
@@ -329,7 +344,8 @@ class PowerParser:
         """varargslist : varargslist COMMA NAME
                        | NAME"""
         if len(p) == 4:
-            p[0] = p[1] + p[3]
+            print (p[1], p[3])
+            p[0] = p[1] + [p[3]]
         else:
             p[0] = [p[1]]
 
@@ -402,11 +418,14 @@ class PowerParser:
                         self.uninit_regs.add(name)
                     autoassign = (name not in self.declared_vars and
                                   name not in self.special_regs)
-            elif isinstance(p[1], ast.Call) and p[1].func.id in ['GPR', 'SPR']:
+            elif isinstance(p[1], ast.Call) and p[1].func.id in \
+                            ['GPR', 'FPR', 'SPR']:
                 print(astor.dump_tree(p[1]))
                 # replace GPR(x) with GPR[x]
-                idx = p[1].args[0]
-                p[1] = ast.Subscript(p[1].func, idx, ast.Load())
+                idx = p[1].args[0].id
+                ridx = ast.Name("_%s" % idx, ast.Load())
+                p[1] = ast.Subscript(p[1].func, ridx, ast.Load())
+                self.read_regs.add(idx)  # add to list of regs to read
             elif isinstance(p[1], ast.Call) and p[1].func.id == 'MEM':
                 print("mem assign")
                 print(astor.dump_tree(p[1]))
@@ -419,7 +438,11 @@ class PowerParser:
             else:
                 print("help, help")
                 print(astor.dump_tree(p[1]))
-            print("expr assign", name, p[1])
+            print("expr assign", name, p[1], "to", p[3])
+            if isinstance(p[3], ast.Name):
+                toname = p[3].id
+                if toname in self.gprs:
+                    self.read_regs.add(toname)
             if name and name in self.gprs:
                 self.write_regs.add(name)  # add to list of regs to write
             p[0] = Assign(autoassign, name, p[1], p[3], iea_mode)
@@ -670,10 +693,10 @@ class PowerParser:
             print(astor.dump_tree(p[1]))
             print("power dump trailerlist")
             print(astor.dump_tree(p[2]))
-            p[0] = apply_trailer(p[1], p[2])
+            p[0] = apply_trailer(p[1], p[2], self.read_regs)
             if isinstance(p[1], ast.Name):
                 name = p[1].id
-                if name in ['RA', 'RS', 'RB', 'RC', 'RT']:
+                if name in regs + fregs:
                     self.read_regs.add(name)
 
     def p_atom_name(self, p):
@@ -734,8 +757,11 @@ class PowerParser:
                     p[2].left.id in self.gprs:
                 rid = p[2].left.id
                 self.read_regs.add(rid)  # add to list of regs to read
-                # create special call to GPR.getz
-                gprz = ast.Name("GPR", ast.Load())
+                # create special call to GPR.getz or FPR.getz
+                if rid in fregs:
+                    gprz = ast.Name("FPR", ast.Load())
+                else:
+                    gprz = ast.Name("GPR", ast.Load())
                 # get testzero function
                 gprz = ast.Attribute(gprz, "getz", ast.Load())
                 # *sigh* see class GPR.  we need index itself not reg value
@@ -849,7 +875,8 @@ class PowerParser:
 
 class GardenSnakeParser(PowerParser):
     def __init__(self, lexer=None, debug=False, form=None, incl_carry=False):
-        self.sd = create_pdecode()
+        if form is not None:
+            self.sd = create_pdecode()
         PowerParser.__init__(self, form, incl_carry)
         self.debug = debug
         if lexer is None:
