@@ -181,6 +181,14 @@ class ALUHelpers:
             spr2_data = sim.spr[spr2_sel].value
             res['fast2'] = spr2_data
 
+    def get_sim_fast_spr3(res, sim, dec2):
+        fast3_en = yield dec2.e.read_fast3.ok
+        if fast3_en:
+            fast3_sel = yield dec2.e.read_fast3.data
+            spr3_sel = fast_reg_to_spr(fast3_sel)
+            spr3_data = sim.spr[spr3_sel].value
+            res['fast3'] = spr3_data
+
     def get_sim_cr_a(res, sim, dec2):
         cridx_ok = yield dec2.e.read_cr1.ok
         if cridx_ok:
@@ -293,6 +301,10 @@ class ALUHelpers:
         if 'fast2' in inp:
             yield alu.p.data_i.fast2.eq(inp['fast2'])
 
+    def set_fast_spr3(alu, dec2, inp):
+        if 'fast3' in inp:
+            yield alu.p.data_i.fast3.eq(inp['fast3'])
+
     def set_cr_a(alu, dec2, inp):
         if 'cr_a' in inp:
             yield alu.p.data_i.cr_a.eq(inp['cr_a'])
@@ -334,6 +346,11 @@ class ALUHelpers:
         if spr2_valid:
             res['fast2'] = yield alu.n.data_o.fast2.data
 
+    def get_fast_spr3(res, alu, dec2):
+        spr3_valid = yield alu.n.data_o.fast3.ok
+        if spr3_valid:
+            res['fast3'] = yield alu.n.data_o.fast3.data
+
     def get_cia(res, alu, dec2):
         res['cia'] = yield alu.p.data_i.cia
 
@@ -406,6 +423,14 @@ class ALUHelpers:
             cridx = yield dec2.e.write_cr.data
             res['cr_a'] = sim.crl[cridx].get_range().value
 
+    def get_wr_fast_spr3(res, sim, dec2):
+        ok = yield dec2.e.write_fast3.ok
+        if ok:
+            spr_num = yield dec2.e.write_fast3.data
+            spr_num = fast_reg_to_spr(spr_num)
+            spr_name = spr_dict[spr_num].SPR
+            res['fast3'] = sim.spr[spr_name].value
+
     def get_wr_fast_spr2(res, sim, dec2):
         ok = yield dec2.e.write_fast2.ok
         if ok:
@@ -502,6 +527,13 @@ class ALUHelpers:
             print(f"expected {expected:x}, actual: {alu_out:x}")
             dut.assertEqual(expected, alu_out, msg)
 
+    def check_fast_spr3(dut, res, sim_o, msg):
+        if 'fast3' in res:
+            expected = sim_o['fast3']
+            alu_out = res['fast3']
+            print(f"expected {expected:x}, actual: {alu_out:x}")
+            dut.assertEqual(expected, alu_out, msg)
+
     def check_int_o1(dut, res, sim_o, msg):
         if 'o1' in res:
             expected = sim_o['o1']
